package de.hsbo.fbv.bmg.graphics.transforms.demos;


import java.awt.geom.Point2D;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import de.hsbo.fbv.bmg.graphics.transforms.BilinearTransform;
import de.hsbo.fbv.bmg.graphics.transforms.TransformFactory;



public class GeoreferenceDemo {
	
	public static void main(String[] args) {
		get4ParTrafo();
		
	}
	
	public static void get4ParTrafo() {
		Point2D[] src = generatePoints(10, 1000, 1000);
		BilinearTransform bt = new BilinearTransform();
		bt.rotate(Math.toRadians(10), 500, 500);
		bt.translate(200, 400);
		bt.scale(2, 2);
		System.out.println(bt);
		
		Point2D[] dst = transformRandom(src, bt, 0.1);
		Map<Point2D, Point2D> map = getMap(src, dst);
		BilinearTransform georef = TransformFactory.create4Parameter(map);
		System.out.println(georef);
		
		Point2D[] geodst = transform(src, georef);
		printStatistics(dst, geodst);
	}

	public static Point2D[] generatePoints(int n, double dx, double dy) {
		Point2D[] points = new Point2D[n];
		points[0] = new Point2D.Double( 0, 0);
		points[1] = new Point2D.Double(dx, 0);
		points[2] = new Point2D.Double( 0, dy);
		points[3] = new Point2D.Double(dx, dy);
		for(int i=4; i<points.length; i++) {
			points[i] = new Point2D.Double(Math.random()*dx, Math.random()*dy);
		}
		return points;
	}
	
	public static Point2D[] transform(Point2D[] src, BilinearTransform bt) {
		Point2D[] dst = new Point2D[src.length];
		for(int i=0; i<src.length; i++)
			dst[i] = bt.transform(src[i]);
		return dst;
	}
	
	public static Point2D[] transformRandom(Point2D[] src, BilinearTransform bt, double rand) {
		Point2D[] dst = new Point2D[src.length];
		for(int i=0; i<src.length; i++) {
			dst[i] = bt.transform(src[i]);
			double rx = (Math.random()-Math.random())*rand;
			double ry = (Math.random()-Math.random())*rand;
			dst[i].setLocation(dst[i].getX()+rx, dst[i].getY()+ry);
		}
		return dst;
	}
	
	public static Map getMap(Point2D[] src, Point2D[] dst) {
		Map<Point2D, Point2D> map = new HashMap<Point2D, Point2D>();
		for(int i=0; i<src.length; i++) {
			map.put(src[i], dst[i]);
		}
		return map;
	}
	
	public static void printStatistics(Point2D[] orig, Point2D[] trans) {
		double sigmaX = 0;
		double sigmaY = 0;
		double c = 1.e+3;
		for(int i=0; i<orig.length; i++) {
			double dx = Math.rint((orig[i].getX()-trans[i].getX())*c)/c;
			double dy = Math.rint((orig[i].getY()-trans[i].getY())*c)/c;
			sigmaX = sigmaX + dx * dx;
			sigmaY = sigmaY + dy * dy;
			System.out.println(dx + " " + dy);
		}
		sigmaX = Math.sqrt(sigmaX/orig.length);
		sigmaY = Math.sqrt(sigmaY/orig.length);
		System.out.println("\n"+sigmaX + " " + sigmaY);
	}
	
}
