package de.hsbo.fbv.bmg.graphics.transforms.demos;


import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.Ellipse2D;
import java.awt.Color;

import javax.swing.JPanel;
import javax.swing.JFrame;


import java.awt.Rectangle;
import java.awt.GridBagLayout;
import javax.swing.JLabel;
import javax.swing.JSlider;

import de.hsbo.fbv.bmg.graphics.transforms.Point2hD;
import de.hsbo.fbv.bmg.graphics.transforms.Shape2hD;
import de.hsbo.fbv.bmg.graphics.transforms.Transform2hD;
import de.hsbo.fbv.bmg.graphics.transforms.TransformFactory;


public class Pseudo3DDemoFrame extends JFrame {

	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private JPanel jPanel = null;
	
	// Manuell eingefuegter Quelltext
	Shape2hD[] shapes =  SpatialDataGenerator.createGrid2hd(100, 20, 20); // Geodaten
	Transform2hD[] trafo = new Transform2hD[2]; // Transformation  //  @jve:decl-index=0:
	double m = 100; 	// Mastab
	double decl = 0; 	// Neigung (declination)
	double azim = 0; 	// Azimuth
	double dist = 100; 	// Abstand / Hhe / Flughhe
	double dx = 0;		// Verschiebung x
	double dy = 0;		// Verschiebung y
	double dz = 0;		// Verschiebung z
	double mh = 1;		// Hhenmastab / berhhung
	double baseline = dist*dist/300;	// Abstand der Stereobildpaare
	private JLabel jLabelHeight = null;
	private JSlider jSliderHeight = null;
	private JLabel jLabelDecl = null;
	private JSlider jSliderDecl = null;
	private JLabel jLabelAzim = null;
	private JSlider jSliderAzim = null;
	private JLabel jLabelM = null;
	private JSlider jSliderM = null;
	private JLabel jLabelX = null;
	private JLabel jLabelY = null;
	private JSlider jSliderX = null;
	private JSlider jSliderY = null;
	private JLabel jLabelMH = null;
	private JSlider jSliderMH = null;
	private JLabel jLabelZ = null;
	private JSlider jSliderZ = null;
	

	/**
	 * This is the default constructor
	 */
	public Pseudo3DDemoFrame() {
		super();
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(800, 600);
		this.setContentPane(getJContentPane());
		this.setTitle("Pseudo 3D Demo");
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jLabelZ = new JLabel();
			jLabelZ.setBounds(new Rectangle(15, 154, 160, 16));
			jLabelZ.setText("Verschiebung z");
			jLabelMH = new JLabel();
			jLabelMH.setBounds(new Rectangle(15, 494, 160, 16));
			jLabelMH.setText("berhhung");
			jLabelY = new JLabel();
			jLabelY.setBounds(new Rectangle(15, 86, 160, 16));
			jLabelY.setText("Verschiebung y");
			jLabelX = new JLabel();
			jLabelX.setBounds(new Rectangle(15, 18, 160, 16));
			jLabelX.setText("Verschiebung x");
			jLabelM = new JLabel();
			jLabelM.setBounds(new Rectangle(15, 426, 160, 16));
			jLabelM.setText("Mastab");
			jLabelAzim = new JLabel();
			jLabelAzim.setBounds(new Rectangle(15, 358, 160, 16));
			jLabelAzim.setText("Azimuth");
			jLabelDecl = new JLabel();
			jLabelDecl.setBounds(new Rectangle(15, 290, 160, 16));
			jLabelDecl.setText("Neigung");
			jLabelHeight = new JLabel();
			jLabelHeight.setBounds(new Rectangle(15, 222, 160, 16));
			jLabelHeight.setText("Abstand");
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.add(getJPanel(), null);
			jContentPane.add(jLabelHeight, null);
			jContentPane.add(getJSliderHeight(), null);
			jContentPane.add(jLabelDecl, null);
			jContentPane.add(getJSliderDecl(), null);
			jContentPane.add(jLabelAzim, null);
			jContentPane.add(getJSliderAzim(), null);
			jContentPane.add(jLabelM, null);
			jContentPane.add(getJSliderM(), null);
			jContentPane.add(jLabelX, null);
			jContentPane.add(jLabelY, null);
			jContentPane.add(getJSliderX(), null);
			jContentPane.add(getJSliderY(), null);
			jContentPane.add(jLabelMH, null);
			jContentPane.add(getJSliderMH(), null);
			jContentPane.add(jLabelZ, null);
			jContentPane.add(getJSliderZ(), null);
		}
		return jContentPane;
	}


	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanel() {
		if (jPanel == null) {
			jPanel = new JPanel() {
				public void paint(Graphics g) {
					Graphics2D g2 = (Graphics2D)g;
					g.setColor(Color.WHITE);
					g.fillRect(0, 0, this.getWidth(), this.getHeight());
					g.setColor(new Color(255,255,0,64));
					g.fillRect(0, 0, this.getWidth(), this.getHeight());
					g2.translate(this.getWidth()/2, this.getHeight()/2);
					g2.scale(m, -m);
					g2.setStroke(new BasicStroke((float)(1/m)));
					trafo = TransformFactory.createPseudo3D(
							mh, baseline, dx, dy, dz, dist, decl, azim);
					for(int i=0; i<shapes.length; i++) {
						if(shapes[i]!=null) {
							// grn
							g2.setColor(new Color(0,255,255,64));
							Shape sg = (shapes[i].toShape2hD(trafo[0])).toShape();
							if(sg!=null)
								g2.draw(sg);
							// rot
							g2.setColor(new Color(255,0,0,64));
							Shape sr = (shapes[i].toShape2hD(trafo[1])).toShape();
							if(sr!=null)
								g2.draw(sr);
						}
					}
					
				}
			};
			jPanel.setLayout(new GridBagLayout());
			jPanel.setBounds(new Rectangle(212, 2, 560, 560));
		}
		return jPanel;
	}

	/**
	 * This method initializes jSliderHeight	
	 * 	
	 * @return javax.swing.JSlider	
	 */
	private JSlider getJSliderHeight() {
		if (jSliderHeight == null) {
			jSliderHeight = new JSlider();
			jSliderHeight.setBounds(new Rectangle(15, 256, 160, 16));
			jSliderHeight.setValue((int)(dist/5));
			jSliderHeight.addMouseListener(new java.awt.event.MouseAdapter() {
				public void mouseReleased(java.awt.event.MouseEvent e) {
					dist = jSliderHeight.getValue()*5;
					jLabelHeight.setText("Abstand: "+ dist);
					repaint();
				}
			});
		}
		return jSliderHeight;
	}

	/**
	 * This method initializes jSliderDecl	
	 * 	
	 * @return javax.swing.JSlider	
	 */
	private JSlider getJSliderDecl() {
		if (jSliderDecl == null) {
			jSliderDecl = new JSlider();
			jSliderDecl.setBounds(new Rectangle(15, 324, 160, 16));
			jSliderDecl.setValue(0);
			jSliderDecl.addMouseListener(new java.awt.event.MouseAdapter() {
				public void mouseReleased(java.awt.event.MouseEvent e) {
					decl = jSliderDecl.getValue()*0.9;
					decl = Math.rint(decl*10)/10;
					jLabelDecl.setText("Neigung: "+decl+"");
					repaint();
				}
			});
		}
		return jSliderDecl;
	}

	/**
	 * This method initializes jSliderAzim	
	 * 	
	 * @return javax.swing.JSlider	
	 */
	private JSlider getJSliderAzim() {
		if (jSliderAzim == null) {
			jSliderAzim = new JSlider();
			jSliderAzim.setBounds(new Rectangle(15, 392, 160, 16));
			jSliderAzim.setValue(0);
			jSliderAzim.addMouseListener(new java.awt.event.MouseAdapter() {
				public void mouseReleased(java.awt.event.MouseEvent e) {
					azim = jSliderAzim.getValue()*3.6;
					azim = Math.rint(azim*10)/10;
					jLabelAzim.setText("Azimuth: "+azim+"");
					repaint();
				}
			});
		}
		return jSliderAzim;
	}

	/**
	 * This method initializes jSliderM	
	 * 	
	 * @return javax.swing.JSlider	
	 */
	private JSlider getJSliderM() {
		if (jSliderM == null) {
			jSliderM = new JSlider();
			jSliderM.setBounds(new Rectangle(15, 460, 160, 16));
			jSliderM.setValue((int)(m/5));
			jSliderM.addMouseListener(new java.awt.event.MouseAdapter() {
				public void mouseReleased(java.awt.event.MouseEvent e) {
					m = jSliderM.getValue()*5;
					jLabelM.setText("Mastab: "+m);
					repaint();
				}
			});
		}
		return jSliderM;
	}

	/**
	 * This method initializes jSliderX	
	 * 	
	 * @return javax.swing.JSlider	
	 */
	private JSlider getJSliderX() {
		if (jSliderX == null) {
			jSliderX = new JSlider();
			jSliderX.setBounds(new Rectangle(15, 52, 160, 16));
			jSliderX.addMouseListener(new java.awt.event.MouseAdapter() {
				public void mouseReleased(java.awt.event.MouseEvent e) {
					dx = (jSliderX.getValue()-50)*20;
					jLabelX.setText("Verschiebung x: "+dx);
					repaint();
				}
			});
		}
		return jSliderX;
	}

	/**
	 * This method initializes jSliderY	
	 * 	
	 * @return javax.swing.JSlider	
	 */
	private JSlider getJSliderY() {
		if (jSliderY == null) {
			jSliderY = new JSlider();
			jSliderY.setBounds(new Rectangle(15, 120, 160, 16));
			jSliderY.addMouseListener(new java.awt.event.MouseAdapter() {
				public void mouseReleased(java.awt.event.MouseEvent e) {
					dy = (jSliderY.getValue()-50)*20;
					jLabelY.setText("Verschiebung y: "+dy);
					repaint();
				}
			});
		}
		return jSliderY;
	}

	/**
	 * This method initializes jSliderMH	
	 * 	
	 * @return javax.swing.JSlider	
	 */
	private JSlider getJSliderMH() {
		if (jSliderMH == null) {
			jSliderMH = new JSlider();
			jSliderMH.setBounds(new Rectangle(15, 528, 160, 16));
			jSliderMH.setValue((int)(mh/20.0));
			jSliderMH.addMouseListener(new java.awt.event.MouseAdapter() {
				public void mouseReleased(java.awt.event.MouseEvent e) {
					mh = jSliderMH.getValue()/20.0;
					mh = Math.rint(mh*10)/10;
					jLabelMH.setText("berhhung: "+mh+":1");
					repaint();
				}
			});
		}
		return jSliderMH;
	}

	/**
	 * This method initializes jSliderZ	
	 * 	
	 * @return javax.swing.JSlider	
	 */
	private JSlider getJSliderZ() {
		if (jSliderZ == null) {
			jSliderZ = new JSlider();
			jSliderZ.setBounds(new Rectangle(15, 188, 160, 16));
			jSliderZ.addMouseListener(new java.awt.event.MouseAdapter() {
				public void mouseReleased(java.awt.event.MouseEvent e) {
					dz = (jSliderZ.getValue()-50)*2;
					jLabelZ.setText("Verschiebung z: "+dz);
					repaint();
				}
			});
		}
		return jSliderZ;
	}


}
