package de.hsbo.fbv.bmg.graphics.viewer.aufgaben;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagLayout;
import java.awt.geom.AffineTransform;
import java.awt.geom.NoninvertibleTransformException;
import java.awt.geom.Path2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;

import javax.swing.JPanel;


/**
 * Einfaches Programm zum Anzeigen von ausgestalteten geometrischen Objekten vom Typ DecoratedShape.
 * Objekte dieser Klasse lassen sich als Komponenten (Beans) in andere Anwendungsfenster einbinden.
 * @author A.Zimmermann, HS Bochum, 2012
 *
 */
public class SimpleShapeViewer extends JPanel {
	
	ArrayList<DecoratedShape> shapes = null;
	AffineTransform at = new AffineTransform();

	private static final long serialVersionUID = 1L;

	/**
	 * This is the default constructor
	 */
	public SimpleShapeViewer() {
		super();
		this.setBackground(Color.WHITE);
		shapes = new ArrayList<DecoratedShape>();
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(300, 200);
		this.setLayout(new GridBagLayout());
		this.addMouseListener(new java.awt.event.MouseAdapter() {
			public void mouseClicked(java.awt.event.MouseEvent e) {
				try {
					// AffineTransform it = ((Graphics2D)(getGraphics())).getTransform().createInverse();
					Point2D ptSrc = new Point2D.Double(e.getX(), e.getY());
					Point2D ptDst = new Point2D.Double();
					at.createInverse().transform(ptSrc, ptDst);
					System.out.println(ptDst);
				} catch (NoninvertibleTransformException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			}
		});
	}

	public void paint(Graphics g) {
		super.paint(g);
		Graphics2D g2 = (Graphics2D)g;
		this.at = TransformFactory.createSimpleWorldToDevice(
				this.getEnvelope(), this.getSize());
		g2.setTransform(at);
		for(int i=0; i<shapes.size(); i++) {
			DecoratedShape ds = shapes.get(i);
			if(ds.isToBeDrawn()) {
				g2.setPaint(ds.getPaint());
				g2.setStroke(ds.getStroke());
				g2.draw(ds.getShape());
			}
			else {
				g2.setPaint(ds.getPaint());
				g2.fill(ds.getShape());	
			}
		}
	}

	public boolean add(DecoratedShape e) {
		return shapes.add(e);
	}

	public DecoratedShape get(int index) {
		return shapes.get(index);
	}

	public boolean remove(Object o) {
		return shapes.remove(o);
	}

	public int number() {
		return shapes.size();
	}
	
	public Rectangle2D getEnvelope() {
		Path2D path = new Path2D.Double();
		for(int i=0; i<shapes.size(); i++)
			path.append(shapes.get(i).getShape(), false);
		return path.getBounds2D();
	}
	
	
	
}
