package de.hsbo.fbv.bmg.graphics.viewer.demos;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;

import javax.swing.JPanel;
import javax.swing.JFrame;


/**
 * Diese Klasse demonstriert die Graphikausgabe mit Hilfe von Double Buffering.<br>
 * Double Buffering dient dazu, den Bildaufbau zu beschleunigen. Dazu wird die
 * Bildgraphik im Hintergrund in einem Rasterbild generiert, und dann erst im 
 * Anzeigefenster prsentiert.
 * @see Knudsen: Java 2D Graphics, O'Reilly, 1999, S.195ff
 * @author A.Zimmermann, HS Bochum, 2011
 * 
 */
public class TestDoubleBuffering extends JFrame {
	
	public static void main(String[] args) {
		JFrame frame = new TestDoubleBuffering();
		frame.setSize(800,700);
		frame.setTitle("TestDoubleBuffering");
		frame.setVisible(true);
	}
	
	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	public BufferedImage bi;
	public Graphics2D g2dbi;
	int dx = 0;
	int dy = 0;
	int mx = 0;
	int my = 0;

	/**
	 * This is the default constructor
	 */
	public TestDoubleBuffering() {
		super();
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setContentPane(getJContentPane());
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.addMouseListener(new java.awt.event.MouseAdapter() {
				public void mousePressed(java.awt.event.MouseEvent e) {
					mx = e.getX();
					my = e.getY();
					repaint();
				}
			});
			jContentPane.addMouseMotionListener(new java.awt.event.MouseMotionAdapter() {   
				public void mouseMoved(java.awt.event.MouseEvent e) {    
					mx = e.getX();
					my = e.getY();
					repaint();
				}
				public void mouseDragged(java.awt.event.MouseEvent e) {
					dx = e.getX() - mx;
					dy = e.getY() - my;
					repaint();
				}
			});
		}
		return jContentPane;
	}
	
	public void paint(Graphics g) {
		this.bi = createImage();
		update(g);
	}

	public void update(Graphics g) {
		super.paint(g);
		Graphics2D g2d = (Graphics2D)g;
		g2d.drawImage(bi, dx, dy, null);
	}
	
	/**
	 * Schreibt die neue Graphik in eine Bitmap
	 * @return
	 */
	public BufferedImage createImage() {
		int width = this.getWidth();
		int height = this.getHeight();
		int type = BufferedImage.TYPE_INT_RGB;
		BufferedImage bi = new BufferedImage(width, height, type);
		this.g2dbi = (Graphics2D)bi.getGraphics();
		this.g2dbi.setColor(Color.WHITE);
		this.g2dbi.fillRect(0, 0, width, height);
		this.g2dbi.setColor(Color.BLACK);
		this.g2dbi.drawLine(0, 0, width, 0);
		for(int i=100; i<width; i+=100) {
			this.g2dbi.drawLine(i, -10, i, 10);
			this.g2dbi.drawString(i+"", i-10, 25);
		}
		this.g2dbi.drawLine(0, 0, 0, height);
		for(int i=100; i<width; i+=100) {
			this.g2dbi.drawLine(-10, i, 10, i);
			this.g2dbi.drawString(i+"", 15, i+5);
		}
		g2dbi.drawLine(100, 100, 400, 400);
		g2dbi.drawRect(100, 100, 300, 300);
		
		g2dbi.drawString(mx + " " + my, 500, 500);
		return bi;
	}
}
