package de.hsbo.fbv.bmg.graphics.viewer.simple;

import javax.swing.JPanel;
import javax.swing.JFrame;

import java.awt.Color;
import java.awt.BorderLayout;
import java.awt.Shape;
import java.awt.Stroke;

/**
 * Simple Shape Frame - Beispiel fr den Einbau von SimpleShapeViewer in ein Anwendungsfenster<br>
 * Die Klasse soll die Verwendung von SimpleShapeViewer als Element in einem Anwendungsfenster 
 * demonstrieren. Sie kann auch dazu verwendet werden, um die Ergebnisse kleiner 
 * Programmierbungen zu geometrischen und graphischen Aufgabenstellungen ohne groen Aufwand 
 * anzeigen zu lassen.<br>
 * Das Grundgerst der Klasse wurde mit Eclipse Visual Editor (jve) generiert und
 * anschlieend manuell ergnzt. Daraus ergab sich folgender Aufbau des Frames:<br>
 * + this-"SimpleShapeFrame"<br>
 *   + JContentPane (mit BorderLayout)<br>
 *     + JPanel (hier ist der SimpleShapeViewer ssv hinterlegt und auf "Center" plaziert)<br>
 * Anmerkung:<br>
 * Damit der SimpleShapeViewer innerhalb des Frames Grafiken entgegennehmen und seine 
 * Darstellungsfarben ndern kann, ist der Betrachter zugleich als Attribut namens ssv eingebunden.
 * Auf diese Weise kann zur Bereitstellung der Zugriffmethoden im Frame vom Delegationsprinzip 
 * Gebrauch gemacht werden.
 * @author Zimmermann, HS Bochum 2009<br>
 * @see SimpleShapeViewer<br>
 */
public class SimpleShapeFrame extends JFrame {

	private static final long serialVersionUID = 1L;

	private JPanel jContentPane = null;
	private JPanel jPanel = null;
	SimpleShapeViewer ssv = null;		// Ergnzung: SimpleShapeViewer als Attribut ssv

	/**
	 * This is the default constructor
	 */
	public SimpleShapeFrame() {
		super();
		this.ssv = new SimpleShapeViewer();
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(300, 200);
		this.setContentPane(getJContentPane());
		this.setTitle("SimpleShapeFrame");
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getJPanel(), BorderLayout.CENTER);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanel() {
		if (jPanel == null) {
			jPanel = ssv;				// genderter Code: hier wird ssv als jPanel eingebunden
			jPanel.setLayout(null);
		}
		return jPanel;
	}

	/**
	 * Grafik-Objekt hinzufgen und Anzeigen lassen.
	 * Methode delegiert an ssv.
	 * 
	 * @param shape
	 * @return void
	 */
	public void add(Shape shape) {
		ssv.add(shape);
	}

	/**
	 * Linienfarbe setzen.
	 * Methode delegiert an ssv.
	 * 
	 * @param drawColor - Linienfarbe
	 * @return void
	 */
	public void setDrawColor(Color drawColor) {
		ssv.setDrawColor(drawColor);
	}

	/**
	 * Fllfarbe setzen.
	 * Methode delegiert an ssv.
	 * 
	 * @param fillColor - Fllfarbe
	 * @return void
	 */
	public void setFillColor(Color fillColor) {
		ssv.setFillColor(fillColor);
	}

	/**
	 * Linienmodus setzen
	 * @param stroke
	 */
	public void setStroke(Stroke stroke) {
		ssv.setStroke(stroke);
	}
	
	

}
